
import java.awt.*;

/**
 * A port is a type of portal.
 */

public abstract class Port extends Portal
{
   /**
    * The constructor.
    */

   public Port(Scene scn)
   {
      super(scn);
   }

   /**
    * Viewport information.
    */

   private double _nView =  400.0;
   private double _nPort = -160.0;

   /**
    * Add perspective.
    */

   protected Coordinates addPerspective(Coordinates coords)
   {
      if (coords == null) return null;

      coords = new Coordinates(coords);

      coords.x = coords.x * Math.abs(_nView / (_nView + _nPort - coords.z));
      coords.y = coords.y * Math.abs(_nView / (_nView + _nPort - coords.z));

      return coords;
   }

   /**
    * Check visibility.
    */

   protected boolean isVisible(Coordinates coords)
   {
      return coords.z < _nView + _nPort;
   }

   /**
    * Scene manipulation methods.
    */

   public void rotateLeft(double f)
   {
      f = Math.PI * (f / 100.0);

      Rotation rot = new Rotation(0.0, 0.0, 0.0, 0.0, 1.0, 0.0, f);

      getScene().transform(rot);

      repaint();
   }

   public void rotateRight(double f)
   {
      f = Math.PI * (f / 100.0);

      Rotation rot = new Rotation(0.0, 0.0, 0.0, 0.0, 1.0, 0.0, -f);

      getScene().transform(rot);

      repaint();
   }

   public void rotateUp(double f)
   {
      f = Math.PI * (f / 100.0);

      Rotation rot = new Rotation(0.0, 0.0, 0.0, 1.0, 0.0, 0.0, -f);

      getScene().transform(rot);

      repaint();
   }

   public void rotateDown(double f)
   {
      f = Math.PI * (f / 100.0);

      Rotation rot = new Rotation(0.0, 0.0, 0.0, 1.0, 0.0, 0.0, f);

      getScene().transform(rot);

      repaint();
   }

   public void translateLeft(double f)
   {
      Translation tr = new Translation(-5.0, 0.0, 0.0);

      getScene().transform(tr);

      repaint();
   }

   public void translateRight(double f)
   {
      Translation tr = new Translation(5.0, 0.0, 0.0);

      getScene().transform(tr);

      repaint();
   }

   public void translateUp(double f)
   {
      Translation tr = new Translation(0.0, -5.0, 0.0);

      getScene().transform(tr);

      repaint();
   }

   public void translateDown(double f)
   {
      Translation tr = new Translation(0.0, 5.0, 0.0);

      getScene().transform(tr);

      repaint();
   }

   /**
    * Viewport manipulation methods.
    */

   public void adjustView(double f)
   {
      _nView += f;
   }

   public void adjustPort(double f)
   {
      _nPort += f;
   }

   /**
    * Ports must define this.
    */

   public abstract void paintScene(Graphics gr);
}
